@@ -6,7 +6,7 @@ use strict;
 use warnings;
 use v5.10;    # needed for state variable
 
-our $VERSION = '0.107'; # VERSION
+our $VERSION = '0.108'; # VERSION
 our $AUTHORITY = 'cpan:NIGELM'; # AUTHORITY
 
 use Moose;
@@ -35,7 +35,7 @@ has password   => ( is => 'ro', isa => 'Str',  required => 1 );
 has orgname    => ( is => 'ro', isa => 'Str',  required => 1, default => 'System' );
 has ssl_verify => ( is => 'ro', isa => 'Bool', default  => 1 );
 has debug      => ( is => 'rw', isa => 'Int',  default  => 0, );
-has timeout => ( is => 'rw', isa => 'Int', default => 120 );    # Defaults to 120 seconds
+has timeout    => ( is => 'rw', isa => 'Int',  default  => 120 );    # Defaults to 120 seconds
 has _debug_trace_directory =>
     ( is => 'ro', isa => Path, coerce => 1, predicate => '_has_debug_trace_directory' );
 
@@ -66,7 +66,10 @@ has ssl_ca_file => (
 
 method _build_ssl_ca_file () { return path( Mozilla::CA::SSL_ca_file() ); }
 method _build_base_url () { return URI->new( sprintf( 'https://%s/', $self->hostname ) ); }
-method _build_default_accept_header () { return ( 'application/*+json;version=' . $self->api_version ); }
+
+method _build_default_accept_header () {
+    return ( 'application/*+json;version=' . $self->api_version );
+}
 method _debug (@parameters) { warn join( '', '# ', @parameters, "\n" ) if ( $self->debug ); }
 
 # ------------------------------------------------------------------------
@@ -262,6 +265,13 @@ has _url_login => (
     clearer => '_clear_url_login',
     builder => '_build_url_login'
 );
+has _url_provider_login => (
+    is      => 'rw',
+    isa     => Uri,
+    lazy    => 1,
+    clearer => '_clear_url_provider_login',
+    builder => '_build_url_provider_login'
+);
 has _raw_version => (
     is      => 'rw',
     isa     => 'HashRef',
@@ -280,6 +290,10 @@ has _raw_version_full => (
 method _build_api_version () { return $self->_raw_version->{Version}; }
 method _build_url_login () { return URI->new( $self->_raw_version->{LoginUrl} ); }
 
+method _build_url_provider_login () {
+    return URI->new( $self->_raw_version->{ProviderLoginUrl} || $self->_raw_version->{LoginUrl} );
+}
+
 method _build_raw_version () {
     my $hash    = $self->_raw_version_full;
     my $version = 0;
@@ -316,7 +330,7 @@ has authorization_token => (
 
 has current_session => (
     is        => 'ro',
-    isa       => 'VMware::vCloudDirector2::Object',
+    isa       => 'Ref',
     clearer   => '_clear_current_session',
     predicate => 'has_current_session',
     lazy      => 1,
@@ -327,18 +341,33 @@ method _build_current_session () {
     my $login_id     = join( '@', $self->username, $self->orgname );
     my $encoded_auth = 'Basic ' . MIME::Base64::encode( join( ':', $login_id, $self->password ) );
     $self->_debug("API: attempting login as: $login_id") if ( $self->debug );
-    my $response =
-        $self->_request( 'POST', $self->_url_login, undef, { Authorization => $encoded_auth } );
+    my $url = ( $self->orgname eq 'System' ) ? $self->_url_provider_login : $self->_url_login;
+    my $session;
+    my $token;
+    if ( $url =~ m|/cloudapi/| ) {
+        my $response = $self->_request(
+            'POST', $url, undef,
+            {   Authorization => $encoded_auth,
+                Accept        => sprintf( "application/json;version=%s", $self->api_version )
+            }
+        );
+        $session = $self->_decode_json_response($response);
+        $token   = $session->{id};
+        $token =~ s/.*://;
+        $token =~ tr/-//d;
+    }
+    else {
+        my $response = $self->_request( 'POST', $url, undef, { Authorization => $encoded_auth } );
+        $token = $response->header('x-vcloud-authorization');
+        ($session) = $self->_build_returned_objects($response);
+    }
 
-    # if we got here then it succeeded, since we throw on failure
-    my $token = $response->header('x-vcloud-authorization');
     $self->_set_authorization_token($token);
     $self->_debug("API: authentication token: $token") if ( $self->debug );
 
     # we also reset the base url to match the login URL
     ## $self->_set_base_url( $self->_url_login->clone->path('') );
 
-    my ($session) = $self->_build_returned_objects($response);
     return $session;
 }
 
@@ -374,7 +403,7 @@ method _build_returned_objects ($response) {
             $mime_type =~ s/;.*//;
             $hash->{type} = $mime_type;
         }
-        my $type = ( $mime_type =~ m|^application/vnd\..*\.(\w+)\+json$| ) ? $1 : $mime_type;
+        my $type       = ( $mime_type =~ m|^application/vnd\..*\.(\w+)\+json$| ) ? $1  : $mime_type;
         my $thing_type = ( substr( $type, -4, 4 ) eq 'List' ) ? substr( $type, 0, -4 ) : $type;
 
         if (    ( $type ne $thing_type )
@@ -473,6 +502,7 @@ method _clear_api_data () {
     $self->_clear_ua;
     $self->_clear_api_version;
     $self->_clear_url_login;
+    $self->_clear_url_provider_login;
     $self->_clear_raw_version;
     $self->_clear_raw_version_full;
     $self->_clear_authorization_token;
@@ -501,7 +531,7 @@ VMware::vCloudDirector2::API - Module to do stuff!
 
 =head1 VERSION
 
-version 0.107
+version 0.108
 
 =head2 Attributes
 
